@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import pytest
 
 pytest.importorskip("numpy")
@@ -6,7 +8,7 @@ import numpy as np
 
 import dask
 import dask.array as da
-from dask.array.core import Array
+from dask.array import Array
 from dask.array.utils import assert_eq
 from dask.multiprocessing import _dumps, _loads
 from dask.utils import key_split
@@ -164,13 +166,11 @@ def test_random_all(sz):
     da.random.geometric(1, size=sz, chunks=3).compute()
     da.random.gumbel(1, size=sz, chunks=3).compute()
     da.random.hypergeometric(1, 2, 3, size=sz, chunks=3).compute()
-    da.random.integers(5, high=15, size=sz, chunks=3).compute()
     da.random.laplace(size=sz, chunks=3).compute()
     da.random.logistic(size=sz, chunks=3).compute()
     da.random.lognormal(size=sz, chunks=3).compute()
     da.random.logseries(0.5, size=sz, chunks=3).compute()
     da.random.multinomial(20, [1 / 6.0] * 6, size=sz, chunks=3).compute()
-    da.random.multivariate_hypergeometric([16, 8, 4], 6, size=sz, chunks=6).compute()
     da.random.negative_binomial(5, 0.5, size=sz, chunks=3).compute()
     da.random.noncentral_chisquare(2, 2, size=sz, chunks=3).compute()
 
@@ -199,7 +199,7 @@ def test_random_all(sz):
 
 def test_RandomState_only_funcs():
     da.random.randint(10, size=5, chunks=3).compute()
-    with pytest.raises(DeprecationWarning):
+    with pytest.warns(DeprecationWarning):
         da.random.random_integers(10, size=5, chunks=3).compute()
     da.random.random_sample(10, chunks=3).compute()
 
