@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import random
 import warnings
 from bisect import bisect_left
@@ -6,15 +8,16 @@ from operator import add, itemgetter
 
 from tlz import accumulate, groupby, pluck, unique
 
+from dask._compatibility import import_optional_dependency
 from dask.core import istask
-from dask.utils import apply, funcname, import_required
+from dask.utils import import_required
 
 
 def BOKEH_VERSION():
-    import bokeh
-    from packaging.version import parse as parse_version
+    bokeh = import_optional_dependency("bokeh")
+    from packaging.version import Version
 
-    return parse_version(bokeh.__version__)
+    return Version(bokeh.__version__)
 
 
 _BOKEH_MISSING_MSG = "Diagnostics plots require `bokeh` to be installed"
@@ -33,98 +36,6 @@ def unquote(expr):
     return expr
 
 
-def pprint_task(task, keys, label_size=60):
-    """Return a nicely formatted string for a task.
-
-    Parameters
-    ----------
-    task:
-        Value within dask graph to render as text
-    keys: iterable
-        List of keys within dask graph
-    label_size: int (optional)
-        Maximum size of output label, defaults to 60
-
-    Examples
-    --------
-    >>> from operator import add, mul
-    >>> dsk = {'a': 1,
-    ...        'b': 2,
-    ...        'c': (add, 'a', 'b'),
-    ...        'd': (add, (mul, 'a', 'b'), 'c'),
-    ...        'e': (sum, ['a', 'b', 5]),
-    ...        'f': (add,),
-    ...        'g': []}
-
-    >>> pprint_task(dsk['c'], dsk)
-    'add(_, _)'
-    >>> pprint_task(dsk['d'], dsk)
-    'add(mul(_, _), _)'
-    >>> pprint_task(dsk['e'], dsk)
-    'sum([_, _, *])'
-    >>> pprint_task(dsk['f'], dsk)
-    'add()'
-    >>> pprint_task(dsk['g'], dsk)
-    '[]'
-    """
-    if istask(task):
-        func = task[0]
-        if func is apply:
-            head = funcname(task[1])
-            tail = ")"
-            args = unquote(task[2]) if len(task) > 2 else ()
-            kwargs = unquote(task[3]) if len(task) > 3 else {}
-        else:
-            if hasattr(func, "funcs"):
-                head = "(".join(funcname(f) for f in func.funcs)
-                tail = ")" * len(func.funcs)
-            else:
-                head = funcname(task[0])
-                tail = ")"
-            args = task[1:]
-            kwargs = {}
-        if args or kwargs:
-            label_size2 = int(
-                (label_size - len(head) - len(tail)) // (len(args) + len(kwargs))
-            )
-            pprint = lambda t: pprint_task(t, keys, label_size2)
-        if args:
-            if label_size2 > 5:
-                args = ", ".join(pprint(t) for t in args)
-            else:
-                args = "..."
-        else:
-            args = ""
-        if kwargs:
-            if label_size2 > 5:
-                kwargs = ", " + ", ".join(
-                    f"{k}={pprint(v)}" for k, v in sorted(kwargs.items())
-                )
-            else:
-                kwargs = ", ..."
-        else:
-            kwargs = ""
-        return f"{head}({args}{kwargs}{tail}"
-    elif isinstance(task, list):
-        if not task:
-            return "[]"
-        elif len(task) > 3:
-            result = pprint_task(task[:3], keys, label_size)
-            return result[:-1] + ", ...]"
-        else:
-            label_size2 = int((label_size - 2 - 2 * len(task)) // len(task))
-            args = ", ".join(pprint_task(t, keys, label_size2) for t in task)
-            return f"[{args}]"
-    else:
-        try:
-            if task in keys:
-                return "_"
-            else:
-                return "*"
-        except TypeError:
-            return "*"
-
-
 def get_colors(palette, funcs):
     """Get a dict mapping funcs to colors from palette.
 
@@ -208,17 +119,11 @@ def visualize(
             f.x_range = top.x_range
             f.title = None
             f.min_border_top = 20
-            if BOKEH_VERSION().major < 3:
-                f.plot_height -= 30
-            else:
-                f.height -= 30
+            f.height -= 30
         for f in figs[:-1]:
             f.xaxis.axis_label = None
             f.min_border_bottom = 20
-            if BOKEH_VERSION().major < 3:
-                f.plot_height -= 30
-            else:
-                f.height -= 30
+            f.height -= 30
         for f in figs:
             f.min_border_left = 75
             f.min_border_right = 75
@@ -300,7 +205,7 @@ def plot_tasks(
         data["width"] = width = [e - s for (s, e) in zip(starts, ends)]
         data["x"] = [w / 2 + s - start_time for (w, s) in zip(width, starts)]
         data["y"] = [id_lk[i] + 1 for i in ids]
-        data["function"] = funcs = [pprint_task(i, dsk, label_size) for i in tasks]
+        data["function"] = funcs = [repr(i) for i in tasks]
         data["color"] = get_colors(palette, funcs)
         data["key"] = [str(i) for i in keys]
 
@@ -494,7 +399,7 @@ def plot_cache(
     if results:
         starts, ends = list(zip(*results))[3:]
         tics = sorted(unique(starts + ends))
-        groups = groupby(lambda d: pprint_task(d[1], dsk, label_size), results)
+        groups = groupby(lambda d: repr(d), results)
         data = {}
         for k, vals in groups.items():
             cnts = dict.fromkeys(tics, 0)
