@@ -106,18 +106,22 @@ significantly on space and computation complexity.
 
 See the function ``inline_functions`` for more information.
 """
+
 from __future__ import annotations
 
 import os
-from collections.abc import Hashable, Mapping, Sequence
+from collections import defaultdict
+from collections.abc import Mapping, Sequence
 from concurrent.futures import Executor, Future
 from functools import partial
 from queue import Empty, Queue
 
 from dask import config
+from dask._task_spec import DataNode, convert_legacy_graph
 from dask.callbacks import local_callbacks, unpack_callbacks
-from dask.core import _execute_task, flatten, get_dependencies, has_tasks, reverse_dict
+from dask.core import flatten, get_dependencies
 from dask.order import order
+from dask.typing import Key
 
 if os.name == "nt":
     # Python 3 windows Queue.get doesn't handle interrupts properly. To
@@ -137,7 +141,7 @@ else:
         return q.get()
 
 
-def start_state_from_dask(dsk, cache=None, sortkey=None):
+def start_state_from_dask(dsk, cache=None, sortkey=None, keys=None):
     """Start state from a dask
 
     Examples
@@ -163,33 +167,57 @@ def start_state_from_dask(dsk, cache=None, sortkey=None):
         cache = config.get("cache", None)
     if cache is None:
         cache = dict()
-    data_keys = set()
-    for k, v in dsk.items():
-        if not has_tasks(dsk, v):
-            cache[k] = v
-            data_keys.add(k)
+    if keys is None:
+        keys = list(set(dsk) - set(cache))
+    dsk = convert_legacy_graph(dsk, all_keys=set(dsk) | set(cache))
+    stack = list(keys)
+    dependencies = defaultdict(set)
+    dependents = defaultdict(set)
+    waiting = defaultdict(set)
+    waiting_data = defaultdict(set)
+    ready_set = set()
+    seen = set()
+    while stack:
+        key = stack.pop()
+        if key in seen:
+            continue
+        seen.add(key)
+        dependents[key]
+        waiting_data[key]
+        dependencies[key]
+        task = dsk.get(key, None)
+        if task is None:
+            continue
+        elif isinstance(task, DataNode):
+            cache[key] = task()
+            dependencies[key]
+            for d in dependents[key]:
+                if d in waiting:
+                    waiting[d].remove(key)
+                    if not waiting[d]:
+                        del waiting[d]
+                        ready_set.add(d)
+                else:
+                    ready_set.add(d)
+        else:
+            _wait = task.dependencies - set(cache)
+            if not _wait:
+                ready_set.add(key)
+            else:
+                waiting[key] = set(_wait)
+            for dep in task.dependencies:
+                dependencies[key].add(dep)
+                dependents[dep].add(key)
+                waiting_data[dep].add(key)
+                stack.append(dep)
 
-    dsk2 = dsk.copy()
-    dsk2.update(cache)
-
-    dependencies = {k: get_dependencies(dsk2, k) for k in dsk}
-    waiting = {k: v.copy() for k, v in dependencies.items() if k not in data_keys}
-
-    dependents = reverse_dict(dependencies)
-    for a in cache:
-        for b in dependents.get(a, ()):
-            waiting[b].remove(a)
-    waiting_data = {k: v.copy() for k, v in dependents.items() if v}
-
-    ready_set = {k for k, v in waiting.items() if not v}
     ready = sorted(ready_set, key=sortkey, reverse=True)
-    waiting = {k: v for k, v in waiting.items() if v}
 
     state = {
-        "dependencies": dependencies,
-        "dependents": dependents,
-        "waiting": waiting,
-        "waiting_data": waiting_data,
+        "dependencies": dict(dependencies),
+        "dependents": dict(dependents),
+        "waiting": dict(waiting),
+        "waiting_data": dict(waiting_data),
         "cache": cache,
         "ready": ready,
         "running": set(),
@@ -221,11 +249,11 @@ def execute_task(key, task_info, dumps, loads, get_id, pack_exception):
     """
     try:
         task, data = loads(task_info)
-        result = _execute_task(task, data)
+        result = task(data)
         id = get_id()
         result = dumps((result, id))
         failed = False
-    except BaseException as e:
+    except BaseException as e:  # noqa: B036
         result = pack_exception(e, dumps)
         failed = True
     return key, result, failed
@@ -421,7 +449,9 @@ def get_async(
         result_flat = {result}
     results = set(result_flat)
 
-    dsk = dict(dsk)
+    if not isinstance(dsk, Mapping):
+        dsk = dsk.__dask_graph__()
+    dsk = convert_legacy_graph(dsk)
     with local_callbacks(callbacks) as callbacks:
         _, _, pretask_cbs, posttask_cbs, _ = unpack_callbacks(callbacks)
         started_cbs = []
@@ -437,7 +467,9 @@ def get_async(
 
             keyorder = order(dsk)
 
-            state = start_state_from_dask(dsk, cache=cache, sortkey=keyorder.get)
+            state = start_state_from_dask(
+                dsk, keys=results, cache=cache, sortkey=keyorder.get
+            )
 
             for _, start_state, _, _, _ in callbacks:
                 if start_state:
@@ -473,7 +505,7 @@ def get_async(
 
                     # Prep args to send
                     data = {
-                        dep: state["cache"][dep] for dep in get_dependencies(dsk, key)
+                        dep: state["cache"][dep] for dep in state["dependencies"][key]
                     }
                     args.append(
                         (
@@ -506,7 +538,7 @@ def get_async(
                                 for dep in get_dependencies(dsk, key)
                             }
                             task = dsk[key]
-                            _execute_task(task, data)  # Re-execute locally
+                            task(data)  # Re-execute locally
                         else:
                             raise_exception(exc, tb)
                     res, worker_id = loads(res_info)
@@ -540,7 +572,7 @@ class SynchronousExecutor(Executor):
         fut = Future()
         try:
             fut.set_result(fn(*args, **kwargs))
-        except BaseException as e:
+        except BaseException as e:  # noqa: B036
             fut.set_exception(e)
         return fut
 
@@ -548,7 +580,7 @@ class SynchronousExecutor(Executor):
 synchronous_executor = SynchronousExecutor()
 
 
-def get_sync(dsk: Mapping, keys: Sequence[Hashable] | Hashable, **kwargs):
+def get_sync(dsk: Mapping, keys: Sequence[Key] | Key, **kwargs):
     """A naive synchronous version of get_async
 
     Can be useful for debugging.
