@@ -1,6 +1,9 @@
+from __future__ import annotations
+
 import pytest
 
 import dask
+from dask._task_spec import Task
 from dask.local import finish_task, get_sync, sortkey, start_state_from_dask
 from dask.order import order
 from dask.utils_test import GetFunctionTestMixin, add, inc
@@ -15,18 +18,18 @@ def test_start_state():
     expected = {
         "cache": {"x": 1, "y": 2},
         "dependencies": {
-            "w": set(["y", "z"]),
-            "x": set([]),
-            "y": set([]),
-            "z": set(["x"]),
+            "w": {"y", "z"},
+            "x": set(),
+            "y": set(),
+            "z": {"x"},
         },
-        "dependents": {"w": set([]), "x": set(["z"]), "y": set(["w"]), "z": set(["w"])},
-        "finished": set([]),
-        "released": set([]),
-        "running": set([]),
+        "dependents": {"w": set(), "x": {"z"}, "y": {"w"}, "z": {"w"}},
+        "finished": set(),
+        "released": set(),
+        "running": set(),
         "ready": ["z"],
-        "waiting": {"w": set(["z"])},
-        "waiting_data": {"x": set(["z"]), "y": set(["w"]), "z": set(["w"])},
+        "waiting": {"w": {"z"}},
+        "waiting_data": {"w": set(), "x": {"z"}, "y": {"w"}, "z": {"w"}},
     }
     assert result == expected
 
@@ -35,7 +38,7 @@ def test_start_state_looks_at_cache():
     dsk = {"b": (inc, "a")}
     cache = {"a": 1}
     result = start_state_from_dask(dsk, cache)
-    assert result["dependencies"]["b"] == set(["a"])
+    assert result["dependencies"]["b"] == {"a"}
     assert result["ready"] == ["b"]
 
 
@@ -54,7 +57,7 @@ def test_start_state_with_tasks_no_deps():
     state = start_state_from_dask(dsk)
     assert list(state["cache"].keys()) == ["b"]
     assert "a" in state["ready"] and "c" in state["ready"]
-    deps = dict((k, set()) for k in "abc")
+    deps = {k: set() for k in "abc"}
     assert state["dependencies"] == deps
     assert state["dependents"] == deps
 
@@ -64,7 +67,7 @@ def test_finish_task():
     sortkey = order(dsk).get
     state = start_state_from_dask(dsk)
     state["ready"].remove("z")
-    state["running"] = set(["z", "other-task"])
+    state["running"] = {"z", "other-task"}
     task = "z"
     result = 2
 
@@ -74,18 +77,18 @@ def test_finish_task():
     assert state == {
         "cache": {"y": 2, "z": 2},
         "dependencies": {
-            "w": set(["y", "z"]),
-            "x": set([]),
-            "y": set([]),
-            "z": set(["x"]),
+            "w": {"y", "z"},
+            "x": set(),
+            "y": set(),
+            "z": {"x"},
         },
-        "finished": set(["z"]),
-        "released": set(["x"]),
-        "running": set(["other-task"]),
-        "dependents": {"w": set([]), "x": set(["z"]), "y": set(["w"]), "z": set(["w"])},
+        "finished": {"z"},
+        "released": {"x"},
+        "running": {"other-task"},
+        "dependents": {"w": set(), "x": {"z"}, "y": {"w"}, "z": {"w"}},
         "ready": ["w"],
         "waiting": {},
-        "waiting_data": {"y": set(["w"]), "z": set(["w"])},
+        "waiting_data": {"w": set(), "y": {"w"}, "z": {"w"}},
     }
 
 
@@ -97,11 +100,7 @@ class TestGetAsync(GetFunctionTestMixin):
 
 
 def test_cache_options():
-    try:
-        from chest import Chest
-    except ImportError:
-        return
-    cache = Chest()
+    cache = {}
 
     def inc2(x):
         assert "y" in cache
@@ -171,10 +170,11 @@ def test_ordering():
 
     get_sync(dsk, "y")
 
-    assert L == sorted(L)
+    assert L == sorted(L, reverse=True)
 
 
 def test_complex_ordering():
+    pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
     from dask.diagnostics import Callback
 
@@ -191,3 +191,25 @@ def test_complex_ordering():
     with Callback(pretask=track_order):
         get_sync(dsk, exp_order[-1])
     assert actual_order == exp_order
+
+
+def test_ensure_calculate_only_whats_needed():
+    counter = 0
+
+    def only_once():
+        nonlocal counter
+        if counter > 0:
+            raise RuntimeError("Should only be called once")
+        counter += 1
+        return 1
+
+    def add(x, y):
+        return x + y
+
+    tasks = [
+        Task("x1", only_once),
+        (t1 := Task("x2", only_once)),
+        Task("y", add, 1, t1.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    assert get_sync(dsk, "y") == 2
